(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('Utility', ['moment', factoryFn]);

  function factoryFn(moment) {
    var utility = this;

    function isNull(value) {
      return value === null || value === undefined || value === '';
    }

    function isTrue(condition) {
      return condition === true;
    }

    function isTruthyAcrossPlatform(value) {
      return value === 1 || value === true || value === 'true';
    }

    function isMatchingFacings(line, contract) {
      if (isNull(contract.IM_Facings_simple_vod__c) || isNull(contract.IM_Facings_simple_vod__c.value)) {
        return true;
      }else {
        return line.Facings_simple_vod__c.value >= contract.IM_Facings_simple_vod__c.value;
      }
    }

    function isMatchingPosition(line, contract) {
      if (isNull(contract.IM_Position_vod__c) || isNull(contract.IM_Position_vod__c.value)) {
        return true;
      }else {
        return line.Position_vod__c.value === contract.IM_Position_vod__c.value;
      }
    }

    function isMatchingLayer(line, contract) {
      if (isNull(contract.IM_Layer_vod__c) || isNull(contract.IM_Layer_vod__c.value)) {
        return true;
      }else {
        return line.Layer_vod__c.value === contract.IM_Layer_vod__c.value;
      }
    }

    function matchingInventoryLines(line, contract) {
      var conditions = [
        isMatchingFacings(line, contract),
        isMatchingPosition(line, contract),
        isMatchingLayer(line, contract)
      ];

      return conditions.every(isTrue);
    }

    function isValidProducts(product) {
      var values = [
        product.Product_Type_vod__c === 'Inventory Monitoring',
        product.Product_Type_vod__c === 'Order' && isTruthyAcrossPlatform(product.Inventory_Monitoring_vod__c)
      ];

      return values.some(isTrue);
    }

    utility.genericQueryErrorHandler = function(deferred) {
      return function(error) {
        deferred.reject(error);
      };
    };

    utility.extractProductId = function (lines) {
      return lines.map(function(line){
        if (line.Product_vod__c) {
          return line.Product_vod__c.value || '';
        }

        return '';
      });
    };

    utility.normalizeArray = function (data, key) {
      if (data && Array.isArray(data[key])) {
        return data[key];
      }

      return [];
    };

    utility.mergeDataFrom = function(candidates, candidateDataField, fieldName, origin, newField) {
      return origin.map(function(obj){
        return candidates.map(function(candidate) {
          if (obj[fieldName] === candidate.Id) {
            obj[newField] = candidate[candidateDataField];
          }
          return obj;
        });
      });
    };

    utility.getCompliantTableHeaders = function(headers, orderDataLabel) {
      var result = [];

      if (headers) {
        result = _.cloneDeep(headers);
        result.push({display: orderDataLabel});
      }

      return result;
    };

    utility.mergeDataToProducts = function(products, invLines, contractLines, orderLines, isFutureOrder) {
      var prodInventoryLines, prodContractLines, prodOrderLines, nameUpdatedInvLines;
      var currentContract;
      var closestOrderLineQuantity;

      return products.map(function(product){
        nameUpdatedInvLines = invLines.map(function(invLine){
          if (invLine.Product_vod__c.value === product.Id) {
            invLine.Product_vod__c.originalDisplay = invLine.Product_vod__c.display;
            invLine.Product_vod__c.display = product.Name;
            invLine.Product_Identifier_vod__c = product.Product_Identifier_vod__c;
          }

          return invLine;
        });

        prodInventoryLines = nameUpdatedInvLines.filter(function(invLine){
          return invLine.Product_vod__c.value === product.Id;
        });

        prodContractLines = contractLines.filter(function(contractLine){
          return contractLine.Product_vod__c.value === product.Id;
        });

        prodOrderLines = orderLines.filter(function(orderLine){
          return orderLine.Product_vod__c === product.Id;
        });

        product.associatedInvLines = prodInventoryLines;
        product.associatedContractLines = prodContractLines;
        product.associatedOrderLines = sortOrderLines(prodOrderLines, isFutureOrder);

        currentContract = product.associatedContractLines[0];
        closestOrderLineQuantity = utility.normalizeFieldWithValue(product.associatedOrderLines[0], 'Quantity_vod__c', null);

        product.associatedInvLines.forEach(function(line) {
          if (currentContract) {
            line.contractComsumerPrice = currentContract.IM_Consumer_Price_vod__c.value;
            line.contractFacingSimple = currentContract.IM_Facings_simple_vod__c.value;
            line.contractInventory = currentContract.IM_Inventory_vod__c.value;
            line.contractLayer = currentContract.IM_Layer_vod__c;
            line.contractPosition = currentContract.IM_Position_vod__c;
          }

          line.orderQuantity = closestOrderLineQuantity;
        });

        return product;
      });
    };

    utility.groupCompliantProducts = function(products) {
      var result = {
        compliant: [],
        nonCompliant: [],
        nonContracted: []
      };

      products.forEach(function(product){
        if (product.associatedContractLines[0]) {
          product.associatedInvLines.forEach(function(invLine) {
            if (matchingInventoryLines(invLine, product.associatedContractLines[0])) {
              result.compliant.push(invLine);
            }else {
              result.nonCompliant.push(invLine);
            }
          });
        }else {
          result.nonContracted = result.nonContracted.concat(product.associatedInvLines);
        }

      });

      return result;
    };

    utility.getCurrentSelectedContractLines = function(products) {
      var result = [];
      var validProducts = products.filter(isValidProducts);

      validProducts.forEach(function(prod){
        result = result.concat(prod.associatedContractLines);
      });

      return result;
    };

    utility.normalizeFieldWithValue = function(object, field, desiredValue) {
      if (object) {
        return object[field];
      }else {
        return desiredValue;
      }
    };

    utility.isNull = isNull;

    function sortOrderLines(orderLines, isAsc) {
      var result;

      result = orderLines.sort(function(a, b) {
        var orderDateSort = moment(a.Order_Date_vod__c, 'YYYY-MM-DD').diff(moment(b.Order_Date_vod__c, 'YYYY-MM-DD'));

        if (orderDateSort === 0) {
          return moment.utc(a.LastModifiedDate).diff(moment.utc(b.LastModifiedDate));
        }

        return orderDateSort;
      });

      if (!isAsc) {
        return result.reverse();
      }

      return result;
    }

    return utility;
  }
})();
